******************************************************************************************************
** PROJECT     : QUOTA                                                                              **
** TITLE       : QUOTA_GUIDE                                                                         *
**             :                                                                                     *
** VERSION     : 1.00                                                                                *
** DATE        : 20070619                                                                            *
** AUTHOR      : PCM                                                                                 *
** HISTORY     : VERSION 1.00 -- Taken from Quota_dev_Sequencer_V006 as first cut on a REAL          *
**             :                 guide sequencer for QUOTA                                           *
**             :                                                                                     *
**             :                 Installed clock channel 30 as a shutter signal                      *
**             :                                                                                     *
** DESCRIPTION : Designed to support initial QUOTA readout and limited guide / shift functionality   *
**             : This code must be run with MCB & CCDAcq board firmware version 4.95                 *
**             :                                                                                    **
******************************************************************************************************
** STA OTA Configuration (Lifted from sta_sequencerV010.asm)                                        **
*                                                                                                    *
*                                                                                                    *
*                            P->   4 22   4 22   4 22                                                *
*                            P->   433    433    433                                                 *
*                            P->    111    111    111                                                *
*                                -----------------------                                             *
*       OTG|SW-2-1|S3-2-1|S3-2-1|S3-2-1|S3-2-1|S3-2-1....                                            *
*        |   <-3 prescan pixels->    parallels dump into S2 and S1                                   *
*        |    OD                                                                                     *
*       _|_   _|                                                                                     *
*     _|   |_|                                                                                       *
* RG-|_      |_                                                                                      *
*      |       |                                                                                     *
*     RD      OS                                                                                     *
*                                                                                                    *
*  NOTE: PSL (standby low) is connected to P3 and P4                                                 *
*        PSH (standby high) is connected to P1 and P2                                                *
*                                                                                                    *
* During integration P1 and P2 are high, so S1 must be low to prevent leakage from the parallels     *
* into the serials.  As part of the transition from the integration state to a serial transfer, S1   * 
* must be set high.  Then S1 will be set low as the first serial charge transfer. Integrate on reset *
* will occur when S2 is high, before S3 (and SW) are set high.                                       *
**                                                                                                  **
******************************************************************************************************

**             Constant definitions
**
** CCD BOARD ROW / COLUMN SELECT MODE SELECT KEYS
DEF #DISABLE_ALL x0001FFFF
DEF #STANDBY_ALL x0002FFFF
DEF #ACTIVE_ALL  x0003FFFF
DEF #ENABLE_R0   x000401FF          * video is enabled for OTA row 0
DEF #ENABLE_R1   x000402FF          * video is enabled for OTA row 1
DEF #ENABLE_R2   x000404FF          * video is enabled for OTA row 2
DEF #ENABLE_R3   x000408FF          * video is enabled for OTA row 3
DEF #ENABLE_R4   x000410FF          * video is enabled for OTA row 4
DEF #ENABLE_R5   x000420FF          * video is enabled for OTA row 5
DEF #ENABLE_R6   x000440FF          * video is enabled for OTA row 6
DEF #ENABLE_R7   x000480FF          * video is enabled for OTA row 7
DEF #ENABLE_GUIDE x0005FFFF         * Enable just the guide regions designated by the guide mask registers
**
** CLOCK BOARD FIRMWARE SHIFT PATTERNS
DEF #CLEAR_SHIFT xFE000000           * -512 vertical shifts i.e. 512 shifts towards serial register
*DEF #CLEAR_SHIFT x02000000           * +512 vertical shifts i.e. 512 shifts towards serial register
DEF #READ_SHIFT  xffff0000           * -1 vertical shift to transfer charge to serial register
*DEF #READ_SHIFT  x00010000           * +1 vertical shift to transfer charge away from serial register
*Here are some examples of other shift patterns
*DEF #READ_SHIFT  x0000FFFE           * -2 vertical shift to transfer charge to serial register
*DEF #READ_SHIFT  x00000002           * +2 vertical shift to transfer charge away from serial register
*DEF #READ_SHIFT  x00010000           * +1 horiz shift to transfer charge toward amplifier
*DEF #READ_SHIFT  xFFFF0000           * -1 horiz shift to transfer charge away from amplifier
*DEF #READ_SHIFT  x00010001           * +1 horiz and +1 vert to transfer charge away from serial register

**********************
** TIMING CONSTANTS **               * Lifted from sta_sequncerV010.asm
**********************
DEF #THETA_OVERLAP   20              * 20 us read parallel phase overlaps (us)
DEF #CLEAR_OVERLAP   20              * 20 us fast clear overlap
DEF #RST_WIDTH       13              * Delay for Reset on time    (clk) 13*50 ns + 3 instructions = 800ns
DEF #RST_SETTLE      13              * Delay for signal to settle (clk) 13*50 ns + 3 instructions = 800ns
DEF #SER_OVERLAP     13              * Delay serial clock overlap (clk) 13*50 ns + 3 instructions = 800ns
DEF #DWELL_PLUS_2    13              * Entered in clk cycles 13*50 ns + 3 instructions = 800ns
DEF #DWELL_PLUS_5    13              * Entered in clk cycles 13*50 ns + 3 instructions = 800ns
DEF #LOGIC_OVERLAP   20              * Logic switching overlap = 20 us
DEF #CELL_SEL_DELAY 250              * Delay between selecting cells = 250 us
DEF #LOGIC_DELAY     16              * Delay after logic switching = 16 ms
DEF #VID_ON_DELAY    36              * Delay between video set (D2=1) and serial clocking = 36 ms
DEF #CDS_DELAY       40              * Delay from microsequencer start to video dump about 2 us
** New constants
DEF #MODE_WAIT       10              * ms Delay to allow CCD Acq Board to mode select OTA Cells
DEF #SHIFT_WAIT      10              * ms Delay to allow CLK Board to complete shifts on selected OTA Cells
DEF #CLEAR_WAIT      10              * ms Delay for clear shift of 500 rows to serial register
DEF #READ_WAIT       10              * ms Delay for parallel transfer to serial register function call
DEF #CLR_OVERLAP      4              * Used to shift serials during clear operation (clk) = 200 ns + 3 instructions = 350ns
**
*******************************
** LOOP REGISTER DEFINITIONS **
*******************************
DEF #ROWS            0              * NUMBER OF ROWS TO READOUT ((DET ROWS - SKIPROW) / ROWBIN)
DEF #COLS            1              * NUMBER OF COLUMNS TO READOUT ((DET COLS - SKIPCOL) / COLBIN)
DEF #ROWBIN          2              * BINNING FACTOR FOR ROWS
DEF #COLBIN          3              * BINNING FACTOR FOR COLS
DEF #GUIDE_BOX_SIZE  4              * NUMBER OF ROWS AND COLUMNS IN GUIDE BOX
DEF #SPARE_5         5              * not used
DEF #SPARE_6         6              * not used
DEF #WIPES           7              * NUMBER OF WIPES PRIOR TO INTEGRATION
DEF #SPARE_8         8              * not used
DEF #SPARE_9         9              * not used
DEF #SPARE_10       10              * not used
DEF #SPARE_11       11              * not used
DEF #SPARE_12       12              * not used
DEF #SPARE_13       13              * not used
DEF #SPARE_14       14              * not used
DEF #SPARE_15       15              * not used
**
***********************************************
** PATTERN POINTER LOOP REGISTER DEFINITIONS **
***********************************************
DEF #CLEAR_ROWS     128             * Number of row transfers to loop through (#CLEAR_ROWS * #CLEAR_BINNING = Effective number of rows cleared) 
DEF #CLEAR_BINNING    4             * Row binning factor for clear array operation
DEF #CLEAR_COLS     512             * Number of columns to shift for each binned row transfer
**********************************
**    JUMP FLAG CONSTANTS       **
** Bit position in CMD Register **
**********************************
DEF #GD_ITC       1                 * Jump on Guide Integration time expired
DEF #SC_ITC       2                 * Jump on Science Integration time expired
DEF #START        3                 * Jump on start exposure flag set
DEF #EXPOSE       4                 * Begin / Maintain Science exposure routine (Start vector bit 0) 
DEF #GUIDE        5                 * Begin / Maintain Guide exposure routine (Start vector bit 1)
DEF #SHIFT        6                 * Begin shift process (Start vector bit 2)
DEF #FOCUS        7                 * Begin focus frame
DEF #TESTBIT      8                 * Spare
DEF #RUN_EXPOSE   9                 * Local flag to run science exposure (Under sequencer control)
DEF #RUN_GUIDE   10                 * Local flag to run guide exposure loop (Under sequencer control)
DEF #SYNC        11                 * Jump on SYNC bit for multiple DHE environment  (Under sequencer control)
DEF #CONTRUN     12                 * Sequencer code continous run enable bit (Usr0Flag)
DEF #FOCUS_FIRST 13                 * Sequencer function flag bit 13
DEF #FOCUS_LAST  14                 * Sequencer function flag bit 14
DEF #FUNC_BUSY   15                 * Sequencer functions 1=Busy 0=Done, use this bit to monitor
                                    * shift logic operations on CLK or CCD boards 
                                    * and mode setting on CCD board.
** Sequencer Command Register constants
**
** EFR Reg constants
DEF #KILL_EXPFLG    x00000001       * Cancel start exposure flag using the EFR register
DEF #STRT_SCI_ITC   x00000002       * Start the Science integration time counter using the EFR register
DEF #STOP_SCI_ITC   x00000004       * Stop the Science integration time counter using the EFR register
DEF #STRT_GD_ITC    x00000008       * Start the Guide integration time counter using the EFR register
DEF #STOP_GD_ITC    x00000010       * Stop the Guide integration time counter using the EFR register
DEF #DISABLE_LSR    x00000100       * Disables automatic board select cancel after an LSR instruction
DEF #ENABLE_LSR     x00000200       * Enables automatic board select cancel after an LSR instruction.
DEF #SET_SYNC_LOW   x00004000       * If enabled as master DHE, set sync bit low after sync delay
DEF #SET_SYNC_HIGH  x00008000       * If enabled as master DHE, set sync signal high immediately
DEF #RESET_BIT_4    x00010000       * Reset bit 4 of the sequencer command register.
DEF #SET_BIT_4      x00020000       * Set bit 4 of the sequencer command register.
DEF #RESET_BIT_5    x00040000       * Reset bit 5 of the sequencer command register.
DEF #SET_BIT_5      x00080000       * Set bit 5 of the sequencer command register.
DEF #RESET_BIT_6    x00100000       * Reset bit 6 of the sequencer command register.
DEF #SET_BIT_6      x00200000       * Set bit 6 of the sequencer command register.
DEF #RESET_BIT_7    x00400000       * Reset bit 7 of the sequencer command register.
DEF #SET_BIT_7      x00800000       * Set bit 7 of the sequencer command register.
DEF #RESET_TESTBIT  x01000000       * Reset bit 8 of the sequencer command register.
DEF #SET_TESTBIT    x02000000       * Set bit 8 of the sequencer command register. -- Used to flag external signal to Oscilloscope
DEF #RESET_BIT_9    x04000000       * Reset bit 9 of the sequencer command register.
DEF #SET_BIT_9      x08000000       * Set bit 9 of the sequencer command register.
DEF #RESET_BIT_10   x10000000       * Reset bit 10 of the sequencer command register.
DEF #SET_BIT_10     x20000000       * Set bit 10 of the sequencer command register.
DEF #KILLANDSYNC    x00004001       * Set sync low and kill start exposure flag
**
** Mode register constants
DEF #RESET       0                  * Mode register constant for board reset operation
DEF #READ        1                  * Mode register constant for read operation
DEF #WRITE16     2                  * Mode register constant for 16 bit write operation
DEF #WRITE32     3                  * Mode register constant for 32 bit write operation
**
**
** LSR Board select assignments
DEF #DSLCT    x00
DEF #MCB      x01                   * DHE Slot 1
DEF #CCDCLK   x02                   * DHE Slot 2
DEF #CCDACQ_1 x04                   * DHE Slot 3
DEF #CCDACQ_2 x08                   * DHE Slot 4
DEF #CCDACQ_3 x10                   * DHE Slot 5
DEF #CCDACQ_4 x20                   * DHE Slot 6
DEF #CCDACQ_ALL x3C                 * DHE Slots 3,4,5,6.
**
**
** CCDCLK BOARD ADDRESS POINTERS
DEF #CLKPORT     x0000
DEF #FBIAS0123   x0001
DEF #FBIAS4567   x0002
**
** CCDACQ BOARD ADDRESS POINTERS
DEF #CDSREG         x0000           * Write raw 32-bit data to the CDS control register
DEF #DOPREG         x0001           * Write raw 32-bit data to the DOP port
DEF #GUIDE_ACTIVE   x0002           * Write a 16-bit mask value to set the referenced guide cells to enabled without video - see note 3. 
DEF #SCIENCE_ACTIVE x0003           * Write a 16-bit mask value to set the referenced science cells to active without video - See note 4. 
DEF #BRST_CMD       x0004
DEF #SEQ_CMD        x0005           * Trigger the Acquisition board micro sequencer - See note 1.
DEF #MODE_CMD       x0006           * OTA Mode control register - Controls the OTA cell states as disabled/enabled/active - See note 2.
DEF #SHIFT_CMD      x0007           * Write a 32-bit word to define an x/y shift for all ot cells in the active state - See note 5.

DEF #MODE_ACTIVE x0007              * Set cells active in mask word written to this address
DEF #TRIG_SHIFT  x1040              * Write to this register to initaiate shift list structure processing
                                    * write a '0' for guide shift list or '1' for science shift list processing  
DEF #PROC_GUIDE     x10400000       * BUT since this is a 16 bit write mode command, we prepend the address
DEF #PROC_SCIENCE   x10400001

********************** CCD Acquisition Firmware Function code Details ***********************************
** Notes                                                                                               **
**    1     Writing to this register triggers the CDS process micro sequencer. The 32-bit word written **
**          to this register controls the channel number to use, the start origin of the sequencer code**
**          and the run length. The formate of the word is:                                            **
**          ----------------------------------------------------------------------------               **
**          | BITS 31 DOWNTO 24 | BITS 23 DOWNTO 16 | BITS 15 DOWNTO 8| BITS 7 DOWNTO 0|               **
**          |-------------------|-------------------|-----------------|----------------|               **
**          |     NOT USED      | CDS CHANNEL MASK  |    LENGTH OF    | STARTING ADDR  |               **
**          |                   |                   |    SEQUENCE     | OF SEQ IN RAM  |               **
**          ----------------------------------------------------------------------------               **
**                                                                                                     **
**    2     THIS FUNCTION CONTROLS THE MODE OF OTA CELLS BY USING THE ROW AND COLUMN                   **
**          SELECT AND D0,D1,D2 SIGNALS ASSIGNED TO THE DOP PORT BITS.                                 **
**          THERE ARE 4 MODES AVAILABLE THAT ARE SET BY WRITING A CODE TO THE MODE                     **
**          REGISTER ALONG WITH A ROW AND COLUMN MASK PATTERN. THE FORMAT FOR THE                      **
**          MODE WORD IS:                                                                              **
**          ----------------------------------------------------------------                           **
**          |    31:19    |    18:16   |     15:8      |        7:0       |                            **
**          |   NOT USED  | MODE  BITS | ROW MASK BITS | COLUMN MASK BITS |                            **
**          ----------------------------------------------------------------                           **
**          MODE  MODE CONSTANT DESCRIPTION                                                            **
**           0    MOD_NOP       DO NOTHING                                                             **
**           1    MOD_DISABLE   USE THE ROW AND COL MASKS TO SET THE CORRESPONDING                     **
**                              CELLS TO DISABLE MODE.                                                 **
**           2    MOD_STANDBY   USE THE ROW AND COL MASKS TO SET THE CORRESPONDING                     **
**                              CELLS TO STANDBY MODE (USED DURING INTEGRATION)                        **
**           3    MOD_ACTIVE    USE THE ROW AND COL MASKS TO SET THE CORRESPONDING                     **
**                              NON-GUIDE CELLS TO ACTIVE WITH VIDEO DISABLED (USED FOR SHIFTING)      **
**           4    MOD_ENABLED   USE THE ROW AND COL MASKS TO SET THE CORRESPONDING                     **
**                              NON-GUIDE CELLS TO ACTIVE WITH VIDEO ENABLED  (USED FOR READOUT)       **
**           5    MOD_GDACTIVE  USE THE ROW AND COL MASKS TO SET THE CORRESPONDING                     **
**                              GUIDE CELLS TO ACTIVE WITH VIDEO DISABLED (USED FOR SHIFTING)          **
**           6    MOD_GDENABLE  USE THE ROW AND COL MASKS TO SET THE CORRESPONDING                     **
**                              GUIDE CELLS TO ACTIVE WITH VIDEO ENABLED  (USED FOR GUIDE READOUT)     **
**                                                                                                     **
**    3     Writing a 16-bit mask word to this register is equivelent to writing a 32-bit word to the  **
**          MODE_CMD register with the mode set to MOD_GDACTIVE. This just enables the sequencer to    **
**          use this mode on masks dynamically stored in the pattern data segment without having to    **
**          prepend the mode code.                                                                     **
**                                                                                                     **
**    4     Writing a 16-bit mask word to this register is equivelent to writing a 32-bit word to the  **
**          MODE_CMD register with the mode set to MOD_ACTIVE. This just enables the sequencer to      **
**          use this mode on masks dynamically stored in the pattern data segment without having to    **
**          prepend the mode code.                                                                     **
**                                                                                                     **
**    5     Write a 32-bit word to move charge on any active OTA cell by x/y amount between -32767 and **
**          32766 cells (a little over the top but you never know what tomorrows detectors may look    **
**          like !). The format of the 32-bit word is:                                                 **
**          ----------------------------------------------------------------------------               **
**          |                 31:16              |                  15:0               |               **
**          |  SIGNED ROW SHIFT VALUE (Y SHIFT)  | SIGNED COLUMN SHIFT VALUE (X SHIFT) |               **
**          ----------------------------------------------------------------------------               **
**           BITS 31:16  TWOS COMPLIMENT VALUE FOR ROW SHIFT (Y-SHIFT).                                **
**           BITS  15:0  TWOS COMPLIMENT VALUE FOR COLUMN SHIFT (X-SHIFT).                             **
**           THE INITIAL AND FINAL CLOCK PATTERN PRESENT ON THE PARALLELS IS                           **
**               P1 = HIGH  = DOP BIT 19                                                               **
**               P2 = HIGH  = DOP BIT 20                                                               **
**               P3 = LOW   = DOP BIT 21                                                               **
**               P4 = LOW   = DOP BIT 22                                                               **
**           THE SELECTED CELLS ARE Y-SHIFTED FIRST, THEN X-SHIFTED.                                   **
**                                                                                                     **
*********************************************************************************************************
                                   
**
** Clock signal definitions generated here
DEF #Ota02_S3   b10000000000000000000000000000000    * Coupled to CLK_OUT_31
DEF #SHUTTER    b01000000000000000000000000000000    * Coupled to CLK_OUT_30
DEF #Ota01_RG   b00100000000000000000000000000000    * Coupled to CLK_OUT_29
DEF #CLK28      b00010000000000000000000000000000    * Coupled to CLK_OUT_28
DEF #CLK27      b00001000000000000000000000000000    * Coupled to CLK_OUT_27
DEF #Ota03_RG   b00000100000000000000000000000000    * Coupled to CLK_OUT_26
DEF #CLK25      b00000010000000000000000000000000    * Coupled to CLK_OUT_25
DEF #CLK24      b00000001000000000000000000000000    * Coupled to CLK_OUT_24
DEF #Ota04_RG   b00000000100000000000000000000000    * Coupled to CLK_OUT_23
DEF #CLK22      b00000000010000000000000000000000    * Coupled to CLK_OUT_22
DEF #Ota02_SW   b00000000001000000000000000000000    * Coupled to CLK_OUT_21
DEF #Ota02_S1   b00000000000100000000000000000000    * Coupled to CLK_OUT_20
DEF #CLK18      b00000000000010000000000000000000    * Coupled to CLK_OUT_19
DEF #CLK17      b00000000000001000000000000000000    * Coupled to CLK_OUT_18
DEF #Ota01_S1   b00000000000000100000000000000000    * Coupled to CLK_OUT_17
DEF #CLK16      b00000000000000010000000000000000    * Coupled to CLK_OUT_16
DEF #CLK15      b00000000000000001000000000000000    * Coupled to CLK_OUT_15
DEF #Ota03_S1   b00000000000000000100000000000000    * Coupled to CLK_OUT_14
DEF #CLK13      b00000000000000000010000000000000    * Coupled to CLK_OUT_13
DEF #CLK12      b00000000000000000001000000000000    * Coupled to CLK_OUT_12
DEF #Ota04_S1   b00000000000000000000100000000000    * Coupled to CLK_OUT_11
DEF #Ota02_RG   b00000000000000000000010000000000    * Coupled to CLK_OUT_10
DEF #Ota02_S2   b00000000000000000000001000000000    * Coupled to CLK_OUT_09
DEF #Ota01_SW   b00000000000000000000000100000000    * Coupled to CLK_OUT_08
DEF #Ota01_S3   b00000000000000000000000010000000    * Coupled to CLK_OUT_07
DEF #Ota01_S2   b00000000000000000000000001000000    * Coupled to CLK_OUT_06
DEF #Ota03_SW   b00000000000000000000000000100000    * Coupled to CLK_OUT_05
DEF #Ota03_S3   b00000000000000000000000000010000    * Coupled to CLK_OUT_04
DEF #Ota03_S2   b00000000000000000000000000001000    * Coupled to CLK_OUT_03
DEF #Ota04_SW   b00000000000000000000000000000100    * Coupled to CLK_OUT_02
DEF #Ota04_S3   b00000000000000000000000000000010    * Coupled to CLK_OUT_01
DEF #Ota04_S2   b00000000000000000000000000000001    * Coupled to CLK_OUT_00
**
**
*********************************************
** SERIAL CLOCK WAVEFORM CONSTRUCTION HERE **
*********************************************
** COMBINE ALL OTA SERIAL CLOCK INTO SUB GROUPS
DEF #S1   #Ota01_S1 or #Ota02_S1 or #Ota03_S1 or #Ota04_S1
DEF #S2   #Ota01_S2 or #Ota02_S2 or #Ota03_S2 or #Ota04_S2
DEF #S3   #Ota01_S3 or #Ota02_S3 or #Ota03_S3 or #Ota04_S3
DEF #SW   #Ota01_SW or #Ota02_SW or #Ota03_SW or #Ota04_SW
DEF #RG   #Ota01_RG or #Ota02_RG or #Ota03_RG or #Ota04_RG
**
** SERIAL CLOCK BIT PATTERNS for S1-S2-S2 clocking (charged transferred to S1 and S2)
DEF #S_STATE_1     #S1                    * This is the state during serial RESET
DEF #S_STATE_12    #S1 OR #S2             * This is the state for parallel transfers
DEF #S_STATE_2     #S2                    * State for integration and OT shift
DEF #S_STATE_23W   #S2 OR #S3 OR #SW
DEF #S_STATE_3W    #S3 OR #SW
DEF #S_STATE_13W   #S1 OR #S3 OR #SW
DEF #S_STATE_1W    #S1 OR #SW
**
** SERIAL CLOCKING VECTORS
DEF #SER_0      #S_STATE_2 OR #RG
DEF #SER_1      #S_STATE_2
DEF #SER_2      #S_STATE_23W
DEF #SER_3      #S_STATE_3W
DEF #SER_4      #S_STATE_13W
DEF #SER_5      #S_STATE_1 
DEF #SER_6      #S_STATE_12
**
DEF #GDE_0      #SER_0 OR #SHUTTER
DEF #GDE_1      #SER_1 OR #SHUTTER
DEF #GDE_2      #SER_2 OR #SHUTTER
DEF #GDE_3      #SER_3 OR #SHUTTER
DEF #GDE_4      #SER_4 OR #SHUTTER
DEF #GDE_5      #SER_5 OR #SHUTTER 
DEF #GDE_6      #SER_6 OR #SHUTTER
**
DEF #CLR_0      #S_STATE_2 OR #RG
DEF #CLR_1      #S_STATE_2 OR #RG
DEF #CLR_2      #S_STATE_23W OR #RG
DEF #CLR_3      #S_STATE_3W OR #RG
DEF #CLR_4      #S_STATE_13W OR #RG
DEF #CLR_5      #S_STATE_1 OR #RG 
DEF #CLR_6      #S_STATE_12 OR #RG
**
** SHUTTER PULSE DEFINITIONS **
DEF #SHUT_OPEN  #SHUTTER OR #S2     * Clock 30 used as a shutter driver ... 
DEF #SHUT_CLOSE #S2
**
** CCD ACQ BOARD MICROSEQUENCER START VECTORS
DEF #CDSCMD0      x0010B00           * Science Readout microcode origin
DEF #CDSCMD1      x0010B10           * Guide Readout microcode origin
**
** TIMING CONTROL DEBUG DEFINITIONS **
DEF #TC_OFF         0x00000000  * Used to set FBIAS1 to Timing_Ctrl_off value
DEF #TC_GD          0x00000004  * Used to set FBIAS1 to Timing_Ctrl_guide value
DEF #TC_SHFT        0x00000008  * Used to set FBIAS1 to Timing_Ctrl_shift value
**
********************************
** PATTERN MEMORY DEFINITIONS **
********************************
** PATTERN MEMORY SEGMENTATION MAP
DEF #PATSEG_CONSTANTS           x0000 * Pattern memory origin - Constant Values
DEF #MODE_SLCT_SEG              x0050 * Mode command words here.
DEF #SERIAL_WAVEFORMS           x0060 * Serial clocking patterns
DEF #PARALLEL_WAVEFORMS         x0070 * Parallel clocking patterns
DEF #SHIFT_PATTERNS             x0080 * Clock board dedicated shift patterns
DEF #CDS_COMMAND_VECTORS        x0090 * CCD Acq board microsequencer command vectors
DEF #PATSEG_DR                  x00A0 * Drain on/off patterns
DEF #CLEAR_WAVEFORMS            x00D0 * Clear process waveforms
DEF #GUIDE_WAVEFORMS            x00F0 * Guide specific serial waveforms (with shutter open)
***************************
** DAC SETTING CONSTANTS ** 
***************************
DEF #DR_ONVALUE     3119        * Sets Drain to 19V
*DEF #DR_OFFVALUE   3119        * Sets Drain to 19V (drains never go off)
*DEF #DR_OFFVALUE   1641        * Sets Drain to 10V
DEF #DR_OFFVALUE    656         * Sets Drain to 4V
DEF #DR0_DACADDR    x02200000
DEF #DR1_DACADDR    x02210000
DEF #DR2_DACADDR    x02220000
DEF #DR3_DACADDR    x02230000
DEF #DR4_DACADDR    x02240000
DEF #DR5_DACADDR    x02250000
DEF #DR6_DACADDR    x02260000
DEF #DR7_DACADDR    x02270000
DEF #DR0_ONCODE     #DR0_DACADDR OR #DR_ONVALUE
DEF #DR0_OFFCODE    #DR0_DACADDR OR #DR_OFFVALUE
DEF #DR1_ONCODE     #DR1_DACADDR OR #DR_ONVALUE
DEF #DR1_OFFCODE    #DR1_DACADDR OR #DR_OFFVALUE
DEF #DR2_ONCODE     #DR2_DACADDR OR #DR_ONVALUE
DEF #DR2_OFFCODE    #DR2_DACADDR OR #DR_OFFVALUE
DEF #DR3_ONCODE     #DR3_DACADDR OR #DR_ONVALUE
DEF #DR3_OFFCODE    #DR3_DACADDR OR #DR_OFFVALUE
DEF #DR4_ONCODE     #DR4_DACADDR OR #DR_ONVALUE
DEF #DR4_OFFCODE    #DR4_DACADDR OR #DR_OFFVALUE
DEF #DR5_ONCODE     #DR5_DACADDR OR #DR_ONVALUE
DEF #DR5_OFFCODE    #DR5_DACADDR OR #DR_OFFVALUE
DEF #DR6_ONCODE     #DR6_DACADDR OR #DR_ONVALUE
DEF #DR6_OFFCODE    #DR6_DACADDR OR #DR_OFFVALUE
DEF #DR7_ONCODE     #DR7_DACADDR OR #DR_ONVALUE
DEF #DR7_OFFCODE    #DR7_DACADDR OR #DR_OFFVALUE

*************************************
** PATTERN MEMORY DATA DEFINITIONS **
*************************************
**
**
ORG PAT #PATSEG_CONSTANTS              * GENERAL CONSTANTS KEPT IN PAT MEM
PAT #KILL_EXPFLG        #KILL_EXPFLG  
PAT #STRT_SCI_ITC       #STRT_SCI_ITC 
PAT #STOP_SCI_ITC       #STOP_SCI_ITC 
PAT #STRT_GD_ITC        #STRT_GD_ITC  
PAT #STOP_GD_ITC        #STOP_GD_ITC  
PAT #DISABLE_LSR        #DISABLE_LSR  
PAT #ENABLE_LSR         #ENABLE_LSR   
PAT #SET_SYNC_LOW       #SET_SYNC_LOW 
PAT #SET_SYNC_HIGH      #SET_SYNC_HIGH
PAT #RESET_EXPOSE       #RESET_BIT_4  
PAT #SET_EXPOSE         #SET_BIT_4    
PAT #RESET_GUIDE        #RESET_BIT_5  
PAT #SET_GUIDE          #SET_BIT_5    
PAT #RESET_SHIFT        #RESET_BIT_6  
PAT #SET_SHIFT          #SET_BIT_6    
PAT #RESET_FOCUS        #RESET_BIT_7  
PAT #SET_FOCUS          #SET_BIT_7    
PAT #RESET_TESTBIT       #RESET_TESTBIT  
PAT #SET_TESTBIT         #SET_TESTBIT    
PAT #RESET_RUN_EXPOSE   #RESET_BIT_9  
PAT #SET_RUN_EXPOSE     #SET_BIT_9    
PAT #RESET_RUN_GUIDE    #RESET_BIT_10 
PAT #SET_RUN_GUIDE      #SET_BIT_10   
PAT #KILLANDSYNC        #KILLANDSYNC
PAT #SHUT_OPEN          #SHUT_OPEN
PAT #SHUT_CLOSE         #SHUT_CLOSE
PAT #TC_OFF             #TC_OFF 
PAT #TC_GD              #TC_GD  
PAT #TC_SHFT            #TC_SHFT
**
ORG PAT #MODE_SLCT_SEG                * CELL MODE SELECTION COMMAND WORDS AND MASKS 
PAT #OTA_ROWS       x00000008        * Number of rows to read out in this list. This list is used by READOUT_ALL
PAT #ENABLE_R0      #ENABLE_R0       * First row to read out - Row 0
PAT #ENABLE_R1      #ENABLE_R1       * Second row to read out - Row 1
PAT #ENABLE_R2      #ENABLE_R2
PAT #ENABLE_R3      #ENABLE_R3
PAT #ENABLE_R4      #ENABLE_R4
PAT #ENABLE_R5      #ENABLE_R5
PAT #ENABLE_R6      #ENABLE_R6
PAT #ENABLE_R7      #ENABLE_R7       * Last row to read out - Row 7
PAT #DISABLE_ALL    #DISABLE_ALL
PAT #STANDBY_ALL    #STANDBY_ALL
PAT #ACTIVE_ALL     #ACTIVE_ALL
PAT #ENABLE_GUIDE   #ENABLE_GUIDE
PAT #PROC_GUIDE     #PROC_GUIDE
PAT #PROC_SCIENCE   #PROC_SCIENCE
**
**
ORG PAT #SERIAL_WAVEFORMS             * SERIAL CLOCKING WAVEFORMS HERE
PAT #SER_0          #SER_0
PAT #SER_1          #SER_1
PAT #SER_2          #SER_2
PAT #SER_3          #SER_3
PAT #SER_4          #SER_4
PAT #SER_5          #SER_5
PAT #SER_6          #SER_6
**
ORG PAT #SHIFT_PATTERNS
PAT #CLEAR_SHIFT    #CLEAR_SHIFT     * Shift pattern used during clear_all routine
PAT #READ_SHIFT     #READ_SHIFT      * Shift charge from parallels into serial

ORG PAT #CDS_COMMAND_VECTORS
PAT #CDSCMD0        #CDSCMD0
PAT #CDSCMD1        #CDSCMD1

ORG PAT #PATSEG_DR
PAT #DR_ON_LC       x00000008
PAT #DR0_ONCODE     #DR0_ONCODE
PAT #DR1_ONCODE     #DR1_ONCODE
PAT #DR2_ONCODE     #DR2_ONCODE
PAT #DR3_ONCODE     #DR3_ONCODE
PAT #DR4_ONCODE     #DR4_ONCODE
PAT #DR5_ONCODE     #DR5_ONCODE
PAT #DR6_ONCODE     #DR6_ONCODE
PAT #DR7_ONCODE     #DR7_ONCODE
PAT #DR_OFF_LC      x00000008
PAT #DR0_OFFCODE    #DR0_OFFCODE
PAT #DR1_OFFCODE    #DR1_OFFCODE
PAT #DR2_OFFCODE    #DR2_OFFCODE
PAT #DR3_OFFCODE    #DR3_OFFCODE
PAT #DR4_OFFCODE    #DR4_OFFCODE
PAT #DR5_OFFCODE    #DR5_OFFCODE
PAT #DR6_OFFCODE    #DR6_OFFCODE
PAT #DR7_OFFCODE    #DR7_OFFCODE

ORG PAT #CLEAR_WAVEFORMS
PAT #CLEAR_ROWS     #CLEAR_ROWS   
PAT #CLEAR_BINNING  #CLEAR_BINNING
PAT #CLEAR_COLS     #CLEAR_COLS   
PAT #CLR_0          #CLR_0
PAT #CLR_1          #CLR_1
PAT #CLR_2          #CLR_2
PAT #CLR_3          #CLR_3
PAT #CLR_4          #CLR_4
PAT #CLR_5          #CLR_5
PAT #CLR_6          #CLR_6

ORG PAT #GUIDE_WAVEFORMS
PAT #GDE_0          #GDE_0
PAT #GDE_1          #GDE_1
PAT #GDE_2          #GDE_2
PAT #GDE_3          #GDE_3
PAT #GDE_4          #GDE_4
PAT #GDE_5          #GDE_5
PAT #GDE_6          #GDE_6



*************************************
* PROGRAM CODE SEGMENT STARTS HERE **
*************************************
** EXECUTIVE ROUTINE
#MAIN   NOP                           * Bug in the sequencer enable logic dictates that this be the first instruction.
        CAL #INIT                     * Establish initial conditions
#EXEC   LMR #WRITE32
        JCB #CMD_GO #START            * IF START flag set, Go sort out who is bothering us.
        JCB #EX_2 #RUN_EXPOSE         * ELSEIF expose is active, check on the science exposure timer flag
        LPP #STOP_SCI_ITC
        LSR #MCB                      * ELSE cancel the integration timer.
#EX_1   JCB #EX_3 #RUN_GUIDE          * IF guiding is active, check on the guide exposure timer flag
        LPP #STOP_GD_ITC
        LSR #MCB                      * ELSE cancel the guide timer.
        JMP #EXEC                     * GOTO exec loop
#EX_2   JCB #READ_SCIENCE #SC_ITC     * IF science integration timer has overflowed, readout the science array.
        JMP #EX_1                     * ELSE GOTO EX_1 - check guiding
#EX_3   JCB #READ_GUIDE #GD_ITC       * IF guide integration timer has overflowed, readout the guide cells.
        JMP #EXEC                     * ELSE GOTO exec loop
        
#CMD_GO LPP #KILL_EXPFLG
        LSR #MCB                      * Kill the START flag in the command register
        JCB #START_SHIFT #SHIFT       * IF SHIFT bit set do a shift operation
        JCB #START_EXPOSE #EXPOSE     * ELSEIF exposure bit set do a science exposure
        LPP #RESET_RUN_EXPOSE
        LSR #MCB                      * ELSE cancel the expose run flag.
#CMD_1  JCB #START_GUIDE #GUIDE       * IF guide bit set do a guide exposure
        LPP #RESET_RUN_GUIDE
        LSR #MCB                      * ELSE cancel the Guide exposure run flag.
        JMP #EXEC                     * GOTO exec loop
        
#START_EXPOSE 
        JCB #CMD_1 #RUN_EXPOSE        * Do not re-start an integration that is already running
        LPP #SET_RUN_EXPOSE
        LSR #MCB                      * Set the expose run flag.
        CAL #CLEAR_ALL
        CAL #OPEN_SHUTTER
        LMR #WRITE32
        LPP #STRT_SCI_ITC
        LSR #MCB                      * Start the science integration timer running.
        LPP #RESET_EXPOSE
        LSR #MCB                      * Reset the command bit
        JMP #CMD_1                    * Go back to see if we need the guide loop running
        
#START_GUIDE 
        JCB #EXEC #RUN_GUIDE          * Do not re-start the guide loop if it is already running
        LPP #SET_RUN_GUIDE
        LSR #MCB                      * Set the guide run flag.
        LPP #STRT_GD_ITC
        LSR #MCB                      * Start the guide integration timer running.
        LPP #RESET_GUIDE
        LSR #MCB                      * Reset the command bit
        CAL #DR_ON                    * Leave drain voltages on while guiding
        JMP #EXEC                     * Go back to drudgery
        
#START_SHIFT 
*     LPP #SET_TESTBIT            ** Pulse the test bit low as a marker 
*     LSR #MCB
        LMR #WRITE16                  * We expect that the shift list structures have been previously downloaded
        LPP #PROC_SCIENCE
        LSR #CCDACQ_ALL               * so trigger the individual ccd acq boards to process the shift lists
        LMR #WRITE32                  * Put the mode back to normal for our colleagues
        CAL #WAITON_BUSY              * Probably not a good idea :-)
        LPP #RESET_SHIFT
        LSR #MCB                      * Reset the command bit
*     LPP #RESET_TESTBIT
*     LSR #MCB                    * Reset test bit        
        JMP #EXEC                     * Go directly back to exec. Illegal to start an exposure or guide loop
                                      * while doing a shift command.                                      
#READ_SCIENCE
        CAL #CLOSE_SHUTTER
        LPP #STOP_SCI_ITC
        LSR #MCB                      * Stop the science integration timer.
        LPP #STOP_GD_ITC
        LSR #MCB                      * Stop the guide integration timer.
        CAL #READOUT_ALL              * Go do a readout of the complete array
        LMR #WRITE32
        LPP #RESET_RUN_GUIDE
        LSR #MCB                      * Cancel the Guide exposure run flag.
        LPP #RESET_RUN_EXPOSE
        LSR #MCB                      * Cancel the expose run flag.
        LPP #RESET_EXPOSE
        LSR #MCB                      * Reset the command bit
        JMP #EXEC

#READ_GUIDE
        LPP #STOP_GD_ITC
        LSR #MCB                      * Stop the guide integration timer.
        CAL #READOUT_GUIDE
        LMR #WRITE32
        LPP #STRT_GD_ITC
        LSR #MCB                      * Start the guide integration timer running.
        JMP #EXEC
**
#INIT   
        CAL #CLOSE_SHUTTER
        CAL #DR_OFF
        RET        
**
#CLEAR_ALL                              * Now clearing all OTA cells at the same time. PCM 20070617 
        LMR #WRITE32                    * Set 32 bit write mode
        CAL #ACTIVATE_ALL               * Bring the whole array into active mode
        LRB #WIPES                      * FOR wipes (set on attribute page)
            LPP #CLEAR_ROWS
            LPB                         * Loop on number of row shifts for clear operation
                LPP #CLR_6              * Set up the serials to dump charge (S1 and S2 high)
                LDA #CLKPORT            * Point to clock port
                LSR #CCDCLK
                LPP #CLEAR_BINNING      * Point to clear row binning value
                LPB                     * and loop on this
                    LPP #READ_SHIFT     * Shift binning value of rows into serials
                    LDA #SHIFT_CMD      * This command done in parallel on all the ccd acquisition cards.
                    LSR #CCDACQ_ALL     * Parallel row is dumped into serial register here for all cells
                    CAL #WAITON_BUSY
                LPE                     * End of clear row binning loop
                CAL #SER_CLEAR          * Move the charge 1 pixel along the serials
          LPE                           * ENDFOR clear rows
        LPE                             * ENDFOR wipes
        CAL #STNDBY_ALL                 * Put the cell(s) into standby ready for next row
        LPP #SER_1                      * Set up the serials for integrate (S2 high)
        LDA #CLKPORT                    * Point to clock port
        LSR #CCDCLK
        RET
**
#SER_CLEAR
        LPP #CLEAR_COLS
        LPB                             * FOR cols (cols in a cell not OTA rows)
            LDA #CLKPORT
            LPP #CLR_0
            LSR #CCDCLK
            DSC #CLR_OVERLAP
            LPP #CLR_1
            LSR #CCDCLK
            DSC #CLR_OVERLAP
            LPP #CLR_2
            LSR #CCDCLK
            DSC #CLR_OVERLAP
            LPP #CLR_3
            LSR #CCDCLK
            DSC #CLR_OVERLAP
            LPP #CLR_4
            LSR #CCDCLK
            DSC #CLR_OVERLAP
            LPP #CLR_5
            LSR #CCDCLK
            DSC #CLR_OVERLAP
            LPP #CLR_6
            LSR #CCDCLK
        LPE                             * ENDFOR cols
        RET
**
* Was called #READ_ALL when lifted from sta_sequencerV010.asm
* Method has changed slightly to take advantage of pattern pointer
* behaviour during calls. When a call is made, the current pattern
* pointer is saved on a stack, upon return the pattern pointer is
* restored to the value before the call was made. 
* Reads each OTA row in sequence.
* Each row is set to active, then the charge is clocked out
* using the READ_ROW routine then the row is set back to
* standby. (There is now easy way to set up a loop for
* this routine either since READ_ROW moves the pattern
* pointer.)
#READOUT_ALL
        CAL #DR_ON
        CAL #STNDBY_ALL             * Start with a fresh array mode
        LPP #OTA_ROWS               * Point to the start of the row read list
        LPB                         * Loop on number of ota rows which is first item in list
        IPP                         * Point to next row enable mask that selects the OTA row
        CAL #ROW_WRAPPER            * for read out
        LPE                         * end of ota row loop
        CAL #DR_OFF
        RET
**
#ROW_WRAPPER
        LDA #MODE_CMD
        LSR #CCDACQ_ALL             * SET SELECTED ROW TO ENABLED
        CAL #WAITON_BUSY            * WAIT FOR THE ENABLE FUNCTION TO FINISH
        CAL #READ_ROW
        CAL #STNDBY_ALL             * Put the cell(s) into standby ready for next row
        RET
**
#READ_ROW                        * Guess where this was lifted from ! 
* Pattern pointer must be set to the row to be read out prior to calling.
* OTA row is set to active.
* Video is enabled
* Nested loop where the number of rows and columns on a single cell
* are used as the loop counters. A row is shifted into the serials
* then the serial register is shifted by the number of columns in
* a cell.
    *First set up the serial for transfer -DGS
        LDA #CLKPORT
        LPP #SER_6      * Set up the serials to dump charge (S1 and S2 high)
        LSR #CCDCLK
    *Now clock one pixel into serials -DGS
        LRB #ROWS                   * FOR rows (rows in a cell not OTA rows)
*           LRB #ROWBIN   AS IS THIS WILL NOT WORK - NEED TO DIVIDE ROWS BY ROWBIN SOMEHOW          
                LPP #READ_SHIFT             * x0000FFFF is -1 vertical shift to transfer charge to serial register
                LDA #SHIFT_CMD
                LSR #CCDACQ_ALL             * Shift the charge down one row
                CAL #WAITON_BUSY 
*           LPE
        *Now clock the serials for readout -DGS
            LRB #COLS                   * FOR columns (columns in a cell not OTA columns)
                LPP #CDSCMD0                *Load the CDS microsequencer trigger word
                LDA #SEQ_CMD
                LSR #CCDACQ_ALL             *Fire the CDS microsequencer
                LDA #CLKPORT
                LPP #SER_0
                LSR #CCDCLK
                DSC #RST_WIDTH              * Reset hold time
                LPP #SER_1
                LSR #CCDCLK
                DSC #RST_SETTLE             * Recovery from reset
                LPP #SER_2
                LSR #CCDCLK
                DSC #SER_OVERLAP
                LPP #SER_3
                LSR #CCDCLK
                DSC #SER_OVERLAP
                LPP #SER_4
                LSR #CCDCLK
                DSC #DWELL_PLUS_2
                LPP #SER_5
                LSR #CCDCLK
                DSC #DWELL_PLUS_5
                LPP #SER_6
                LSR #CCDCLK
                DSC 50
            LPE                         * ENDFOR columns
        LPE                         * ENDFOR rows
        RET
**
#READOUT_GUIDE
        LMR #WRITE32
        LPP #SET_TESTBIT        ** Test bit for measuring guide readout time
        LSR #MCB
        LMR #WRITE16                * Shift the guide regions to the origin of the cell
        LPP #PROC_GUIDE
        LSR #CCDACQ_ALL             * We expect that the shift list structures have been previously downloaded
        LMR #WRITE32                * Put the mode back to normal for our colleagues                                      
        CAL #WAITON_BUSY
*     LPP #SET_TESTBIT            ** Pulse the test bit low as a marker 
*     LSR #MCB
        LPP #ENABLE_GUIDE
        LDA #MODE_CMD
        LSR #CCDACQ_ALL             * SET ALL GUIDE CELLS ACTIVE VIA CCD DOP
        CAL #WAITON_BUSY
     LPP #RESET_TESTBIT
     LSR #MCB                    * Reset test bit        

*        LPP #SET_TESTBIT            ** Pulse the test bit low as a marker 
*        LSR #MCB
              
        *Set up the serial for transfer -DGS
        LDA #CLKPORT
        LPP #GDE_6      * Set up the serials to dump charge (S1 and S2 high)
        LSR #CCDCLK
        LRB #GUIDE_BOX_SIZE         * FOR number of rows in guide box
            LPP #READ_SHIFT
            LDA #SHIFT_CMD          * Shift a row into the serial register.
            LSR #CCDACQ_ALL 
            CAL #WAITON_BUSY
            LRB #GUIDE_BOX_SIZE         * FOR number of columns in guide box
                LPP #CDSCMD0                *Load the CDS microsequencer trigger word
                LDA #SEQ_CMD
                LSR #CCDACQ_ALL             *Fire the CDS microsequencer
                LDA #CLKPORT
                LPP #GDE_0
                LSR #CCDCLK
                DSC #RST_WIDTH              * Reset hold time
                LPP #GDE_1
                LSR #CCDCLK
                DSC #RST_SETTLE             * Recovery from reset
                LPP #GDE_2
                LSR #CCDCLK
                DSC #SER_OVERLAP
                LPP #GDE_3
                LSR #CCDCLK
                DSC #SER_OVERLAP
                LPP #GDE_4
                LSR #CCDCLK
                DSC #DWELL_PLUS_2
                LPP #GDE_5
                LSR #CCDCLK
                DSC #DWELL_PLUS_5
                LPP #GDE_6
                LSR #CCDCLK
                DSC 50
            LPE                         * ENDFOR columns
        LPE                         * ENDFOR rows
        CAL #STNDBY_ALL             * Start with a fresh array mode
        LMR #WRITE16                  * We expect that the shift list structures have been previously downloaded
        LPP #PROC_SCIENCE
        LSR #CCDACQ_ALL               * so trigger the individual ccd acq boards to process the shift lists
        LMR #WRITE32                  * Put the mode back to normal for our colleagues
        RET
**
#OPEN_SHUTTER
        LMR #WRITE32
        LDA #CLKPORT
        LPP #SHUT_OPEN
        LSR #CCDCLK
        RET
**
#CLOSE_SHUTTER
        LMR #WRITE32
        LDA #CLKPORT
        LPP #SHUT_CLOSE
        LSR #CCDCLK
        RET
**
#DEBUG_TC_OFF
        LMR #WRITE32
        LDA #FBIAS4567
        LPP #TC_OFF
        LSR #CCDCLK
        RET
**
#DEBUG_TC_GD
        LMR #WRITE32
        LDA #FBIAS4567
        LPP #TC_GD
        LSR #CCDCLK
        RET
**
#DEBUG_TC_SHFT
        LMR #WRITE32
        LDA #FBIAS4567
        LPP #TC_SHFT
        LSR #CCDCLK
        RET
**
#ACTIVATE_ALL                       * Lifted and modified from sta_sequencer_V010.asm  
        LMR #WRITE32                * SET 32 BIT WRITE MODE
        LPP #ACTIVE_ALL
        LDA #MODE_CMD
        LSR #CCDACQ_ALL             * SET ALL CELLS ACTIVE VIA CCD DOP
        CAL #WAITON_BUSY 
        RET
**
#STNDBY_ALL                       * Lifted and modified from sta_sequencer_V010.asm   
        LMR #WRITE32                * SET 32 BIT WRITE MODE
        LPP #STANDBY_ALL
        LDA #MODE_CMD
        LSR #CCDACQ_ALL             * SET ALL CELLS STANDBY VIA CCD DOP
        CAL #WAITON_BUSY
        DUS 50
        RET
**
#DISABLE_ALL                        * Lifted and modified from sta_sequencer_V010.asm   
        LMR #WRITE32                * SET 32 BIT WRITE MODE
        LPP #DISABLE_ALL
        LDA #MODE_CMD
        LSR #CCDACQ_ALL             * SET ALL CELLS DISABLED VIA CCD DOP
        CAL #WAITON_BUSY 
        RET

#DR_ON
        LMR #WRITE16
        LPP #DR_ON_LC
        LPB
            IPP
            LSR #CCDACQ_ALL
            CAL #WAITON_BUSY 
            DUS 30
        LPE
        LMR #WRITE32
        RET

#DR_OFF
        LMR #WRITE16
        LPP #DR_OFF_LC
        LPB
            IPP
            LSR #CCDACQ_ALL
            CAL #WAITON_BUSY 
            DUS 30
        LPE
        LMR #WRITE32
        RET

#WAITON_BUSY
        JCB #WAITON_BUSY #FUNC_BUSY
        RET
**
** End of sequencer code segment
**
